/* -*- Mode: JavaScript; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set ts=2 et sw=2 tw=80: */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Raindrop.
 *
 * The Initial Developer of the Original Code is
 * the Mozilla Foundation.
 * Portions created by the Initial Developer are Copyright (C) 2011
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *	Anant Narayanan <anant@kix.in>
 *	Shane Caraveo <shanec@mozillamessaging.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const { Cc, Ci, Cm, Cu } = require("chrome");

let tmp = {}
Cu.import("resource://gre/modules/Services.jsm", tmp);
let { Services } = tmp;

function loadStylesheet(win, uritail) {
  var uri = require("self").data.url(uritail);
  let document = win.document;
  let pi = document.createProcessingInstruction("xml-stylesheet", "href=\"" + uri + "\" type=\"text/css\"");
  document.insertBefore(pi, document.firstChild);
  return pi;
}

/* l10n support. See https://github.com/Mardak/restartless/examples/l10nDialogs */

function getString(name, args, plural) {
  let str;
  try {
    str = getString.bundle.GetStringFromName(name);
  } catch (ex) {
    str = getString.fallback.GetStringFromName(name);
  }
  if (args != null) {
    if (typeof args == "string" || args.length == null) args = [args];
    str = str.replace(/%s/gi, args[0]);
    Array.forEach(args, function(replacement, index) {
      str = str.replace(RegExp("%" + (index + 1) + "\\$S", "gi"), replacement);
    });
  }
  return str;
}
getString.init = function(getAlternate) {
  if (typeof getAlternate != "function") getAlternate = function()"en-US";

  function getBundle(locale) {
    let propertyPath = "locale/" + locale + ".properties";
    let propertyFile = require("self").data.url(propertyPath);
    try {
      let uniqueFileSpec = propertyFile + "#" + Math.random();
      let bundle = Services.strings.createBundle(uniqueFileSpec);
      bundle.getSimpleEnumeration();
      return bundle;
    } catch (ex) {}
    return null;
  }

  let locale = Cc["@mozilla.org/chrome/chrome-registry;1"].
  getService(Ci.nsIXULChromeRegistry).getSelectedLocale("global");
  getString.bundle = getBundle(locale) || getBundle(getAlternate(locale));
  getString.fallback = getBundle("en-US");
}

exports.loadStylesheet = loadStylesheet;
exports.getString = getString;
